@@ -34,7 +34,7 @@ PERFORMANCE OF THIS SOFTWARE.
 #include "Xlcint.h"
 #include "Ximint.h"
 
-Public Xic
+Xic
 _XimICOfXICID(
     Xim		  im,
     XICID	  icid)
@@ -48,7 +48,7 @@ _XimICOfXICID(
     return (Xic)0;
 }
 
-Private void
+static void
 _XimProcIMSetEventMask(
     Xim		 im,
     XPointer	 buf)
@@ -60,7 +60,7 @@ _XimProcIMSetEventMask(
     return;
 }
 
-Private void
+static void
 _XimProcICSetEventMask(
     Xic		 ic,
     XPointer	 buf)
@@ -73,7 +73,7 @@ _XimProcICSetEventMask(
     return;
 }
 
-Public Bool
+Bool
 _XimSetEventMaskCallback(
     Xim		 xim,
     INT16	 len,
@@ -98,7 +98,7 @@ _XimSetEventMaskCallback(
     return False;
 }
 
-Private Bool
+static Bool
 _XimSyncCheck(
     Xim          im,
     INT16        len,
@@ -127,7 +127,7 @@ _XimSyncCheck(
     return False;
 }
 
-Public Bool
+Bool
 _XimSync(
     Xim		 im,
     Xic		 ic)
@@ -185,7 +185,7 @@ _XimSync(
     return True;
 }
 
-Public Bool
+Bool
 _XimProcSyncReply(
     Xim		 im,
     Xic		 ic)
@@ -208,22 +208,18 @@ _XimProcSyncReply(
     return True;
 }
 
-Public Bool
+Bool
 _XimRespSyncReply(
     Xic		 ic,
     BITMASK16	 mode)
 {
-    if (mode & XimSYNCHRONUS) /* SYNC Request */ {
-	if (IS_FOCUSED(ic))
-	    MARK_NEED_SYNC_REPLY(ic);
-	else
-	    _XimProcSyncReply((Xim)ic->core.im, ic);
-    }
+    if (mode & XimSYNCHRONUS) /* SYNC Request */
+	MARK_NEED_SYNC_REPLY(ic->core.im);
 
     return True;
 }
 
-Public Bool
+Bool
 _XimSyncCallback(
     Xim		 xim,
     INT16	 len,
@@ -244,7 +240,7 @@ _XimSyncCallback(
     return False;
 }
 
-Private INT16
+static INT16
 _XimSetEventToWire(
     XEvent	*ev,
     xEvent	*event)
@@ -256,7 +252,7 @@ _XimSetEventToWire(
     return sz_xEvent;
 }
 
-Private Bool
+static Bool
 _XimForwardEventCore(
     Xic		 ic,
     XEvent	*ev,
@@ -328,7 +324,7 @@ _XimForwardEventCore(
     return True;
 }
 
-Public Bool
+Bool
 _XimForwardEvent(
     Xic		 ic,
     XEvent	*ev,
@@ -342,7 +338,7 @@ _XimForwardEvent(
     return _XimForwardEventCore(ic, ev, sync);
 }
 
-Private void
+static void
 _XimProcEvent(
     Display		*d,
     Xic			 ic,
@@ -356,11 +352,11 @@ _XimProcEvent(
     ev->xany.serial |= serial << 16;
     ev->xany.send_event = False;
     ev->xany.display = d;
-    MARK_FABLICATED(ic);
+    MARK_FABRICATED(ic->core.im);
     return;
 }
 
-Private Bool
+static Bool
 _XimForwardEventRecv(
     Xim		 im,
     Xic		 ic,
@@ -379,7 +375,7 @@ _XimForwardEventRecv(
     return True;
 }
 
-Public Bool
+Bool
 _XimForwardEventCallback(
     Xim		 xim,
     INT16	 len,
@@ -400,7 +396,7 @@ _XimForwardEventCallback(
     return False;
 }
 
-Private Bool
+static Bool
 _XimRegisterTriggerkey(
     Xim			 im,
     XPointer		 buf)
@@ -447,7 +443,7 @@ _XimRegisterTriggerkey(
     return True;
 }
 
-Public Bool
+Bool
 _XimRegisterTriggerKeysCallback(
     Xim		 xim,
     INT16	 len,
@@ -461,7 +457,7 @@ _XimRegisterTriggerKeysCallback(
     return True;
 }
 
-Public EVENTMASK
+EVENTMASK
 _XimGetWindowEventmask(
     Xic		 ic)
 {
@@ -474,7 +470,7 @@ _XimGetWindowEventmask(
 }
 
 
-Private Bool
+static Bool
 _XimTriggerNotifyCheck(
     Xim          im,
     INT16        len,
@@ -503,7 +499,7 @@ _XimTriggerNotifyCheck(
     return False;
 }
 
-Public Bool
+Bool
 _XimTriggerNotify(
     Xim		 im,
     Xic		 ic,
@@ -571,7 +567,7 @@ _XimTriggerNotify(
     return True;
 }
 
-Private Bool
+static Bool
 _XimRegCommitInfo(
     Xic			 ic,
     char		*string,
@@ -592,7 +588,7 @@ _XimRegCommitInfo(
     return True;
 }
 
-Private void
+static void
 _XimUnregCommitInfo(
     Xic			ic)
 {
@@ -610,7 +606,7 @@ _XimUnregCommitInfo(
     return;
 }
 
-Public void
+void
 _XimFreeCommitInfo(
     Xic			ic)
 {
@@ -619,7 +615,7 @@ _XimFreeCommitInfo(
     return;
 }
 
-Private Bool
+static Bool
 _XimProcKeySym(
     Xic			  ic,
     CARD32		  sym,
@@ -639,7 +635,7 @@ _XimProcKeySym(
     return True;
 }
 
-Private Bool
+static Bool
 _XimProcCommit(
     Xic		  ic,
     BYTE	 *buf,
@@ -663,7 +659,7 @@ _XimProcCommit(
     return True;
 }
 
-Private Bool
+static Bool
 _XimCommitRecv(
     Xim		 im,
     Xic		 ic,
@@ -691,8 +687,10 @@ _XimCommitRecv(
 	    return False;
 
 	if (!(_XimProcCommit(ic, (BYTE *)&buf_s[5],
-			 		(int)buf_s[4], &string, &string_len)))
+					(int)buf_s[4], &string, &string_len))) {
+	    Xfree(keysym);
 	    return False;
+	}
     }
 
     if (!(_XimRegCommitInfo(ic, string, string_len, keysym, keysym_len))) {
@@ -706,7 +704,7 @@ _XimCommitRecv(
 
     (void)_XimRespSyncReply(ic, flag);
 
-    MARK_FABLICATED(ic);
+    MARK_FABRICATED(im);
 
     ev.type = KeyPress;
     ev.send_event = False;
@@ -720,7 +718,7 @@ _XimCommitRecv(
     return True;
 }
 
-Public Bool
+Bool
 _XimCommitCallback(
     Xim		 xim,
     INT16	 len,
@@ -741,7 +739,7 @@ _XimCommitCallback(
     return False;
 }
 
-Public void
+void
 _XimProcError(
     Xim		 im,
     Xic		 ic,
@@ -750,7 +748,7 @@ _XimProcError(
     return;
 }
 
-Public Bool
+Bool
 _XimErrorCallback(
     Xim		 xim,
     INT16	 len,
@@ -779,7 +777,7 @@ _XimErrorCallback(
     return True;
 }
 
-Public Bool
+Bool
 _XimError(
     Xim		 im,
     Xic		 ic,
@@ -823,7 +821,7 @@ _XimError(
     return True;
 }
 
-Private int
+static int
 _Ximctsconvert(
     XlcConv	 conv,
     char	*from,
@@ -889,7 +887,7 @@ _Ximctsconvert(
     return to_cnvlen;
 }
 
-Public int
+int
 _Ximctstombs(XIM xim, char *from, int from_len,
 	     char *to, int to_len, Status *state)
 {
@@ -897,7 +895,7 @@ _Ximctstombs(XIM xim, char *from, int from_len,
 			  from, from_len, to, to_len, state);
 }
 
-Public int
+int
 _Ximctstowcs(
     XIM		 xim,
     char	*from,
@@ -965,7 +963,7 @@ _Ximctstowcs(
     return to_cnvlen;
 }
 
-Public int
+int
 _Ximctstoutf8(
     XIM		 xim,
     char	*from,
@@ -978,7 +976,7 @@ _Ximctstoutf8(
 			  from, from_len, to, to_len, state);
 }
 
-Public int
+int
 _XimProtoMbLookupString(
     XIC			 xic,
     XKeyEvent		*ev,
@@ -1041,7 +1039,7 @@ _XimProtoMbLookupString(
     return ret;
 }
 
-Public int
+int
 _XimProtoWcLookupString(
     XIC			 xic,
     XKeyEvent		*ev,
@@ -1104,7 +1102,7 @@ _XimProtoWcLookupString(
     return ret;
 }
 
-Public int
+int
 _XimProtoUtf8LookupString(
     XIC			 xic,
     XKeyEvent		*ev,
