@@ -1,10 +1,8 @@
-/* $XdotOrg: xc/lib/X11/cmsColNm.c,v 1.3 2005/05/13 22:53:44 sandmann Exp $ */
-/* $Xorg: cmsColNm.c,v 1.3 2000/08/17 19:45:09 cpqbld Exp $ */
 
 /*
  * Code and supporting documentation (c) Copyright 1990 1991 Tektronix, Inc.
  * 	All Rights Reserved
- * 
+ *
  * This file is a component of an X Window System-specific implementation
  * of Xcms based on the TekColor Color Management System.  Permission is
  * hereby granted to use, copy, modify, sell, and otherwise distribute this
@@ -12,10 +10,10 @@
  * that this copyright, permission, and disclaimer notice is reproduced in
  * all copies of this software and in supporting documentation.  TekColor
  * is a trademark of Tektronix, Inc.
- * 
+ *
  * Tektronix makes no representation about the suitability of this software
  * for any purpose.  It is provided "as is" and with all faults.
- * 
+ *
  * TEKTRONIX DISCLAIMS ALL WARRANTIES APPLICABLE TO THIS SOFTWARE,
  * INCLUDING THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
  * PARTICULAR PURPOSE.  IN NO EVENT SHALL TEKTRONIX BE LIABLE FOR ANY
@@ -32,7 +30,6 @@
  *
  *
  */
-/* $XFree86: xc/lib/X11/cmsColNm.c,v 3.11 2003/04/13 19:22:20 dawes Exp $ */
 
 #ifdef HAVE_CONFIG_H
 #include <config.h>
@@ -43,6 +40,7 @@
 #include <sys/stat.h>
 #include <stdio.h>
 #include <ctype.h>
+#include <limits.h>
 #define XK_LATIN1
 #include <X11/keysymdef.h>
 #include "Cv.h"
@@ -56,7 +54,7 @@ static Status LoadColornameDB(void);
  *		#define declarations local to this package.
  */
 #ifndef XCMSDB
-#define XCMSDB  "/usr/lib/X11/Xcms.txt"
+#define XCMSDB  XCMSDIR "/Xcms.txt"
 #endif
 
 #ifndef isgraph
@@ -187,7 +185,7 @@ _XcmsParseColorString(
  *	DESCRIPTION
  *		Assuming color_string contains a numerical string color
  *		specification, attempts to parse a string into an
- *		XcmsColor structure.		
+ *		XcmsColor structure.
  *
  *	RETURNS
  *		0 if failed; otherwise non-zero.
@@ -257,7 +255,7 @@ FirstCmp(const void *p1, const void *p2)
  *
  */
 {
-    return(strcmp(((XcmsPair *)p1)->first, ((XcmsPair *)p2)->first));
+    return(strcmp(((const XcmsPair *)p1)->first, ((const XcmsPair *)p2)->first));
 }
 
 
@@ -470,7 +468,7 @@ Retry:
 	pair->flag = CYCLE;
 	return(XcmsFailure);
     }
-	    
+
     if (_XcmsParseColorString(ccc, pair->second, pColor) == XcmsSuccess) {
 	/* f2 contains a numerical string specification */
 	return(XcmsSuccess);
@@ -545,7 +543,10 @@ stringSectionSize(
     char *pBuf;
     char *f1;
     char *f2;
-    int i;
+    size_t i;
+
+    unsigned int numEntries = 0;
+    unsigned int sectionSize = 0;
 
     *pNumEntries = 0;
     *pSectionSize = 0;
@@ -570,7 +571,7 @@ stringSectionSize(
 	return(XcmsFailure);
     }
 
-    while((pBuf = fgets(buf, XCMSDB_MAXLINELEN, stream)) != NULL) {
+    while((fgets(buf, XCMSDB_MAXLINELEN, stream)) != NULL) {
 	if ((sscanf(buf, "%s", token)) && (strcmp(token, END_TOKEN) == 0)) {
 	    break;
 	}
@@ -579,26 +580,37 @@ stringSectionSize(
 	    return(XcmsFailure);
 	}
 
-	(*pNumEntries)++;
+	numEntries++;
+	if (numEntries >= INT_MAX)
+	    return(XcmsFailure);
 
-	(*pSectionSize) += (i = strlen(f1)) + 1;
+	i = strlen(f1);
+	if (i >= INT_MAX - sectionSize)
+	    return(XcmsFailure);
+	sectionSize += i + 1;
 	for (; i; i--, f1++) {
 	    /* REMOVE SPACES FROM COUNT */
 	    if (isspace(*f1)) {
-		(*pSectionSize)--;
+		sectionSize--;
 	    }
 	}
 
-	(*pSectionSize) += (i = strlen(f2)) + 1;
+	i = strlen(f2);
+	if (i >= INT_MAX - sectionSize)
+	    return(XcmsFailure);
+	sectionSize += i + 1;
 	for (; i; i--, f2++) {
 	    /* REMOVE SPACES FROM COUNT */
 	    if (isspace(*f2)) {
-		(*pSectionSize)--;
+		sectionSize--;
 	    }
 	}
 
     }
 
+    *pNumEntries = (int) numEntries;
+    *pSectionSize = (int) sectionSize;
+
     return(XcmsSuccess);
 }
 
@@ -654,7 +666,7 @@ ReadColornameDB(
      * Process lines between START_TOKEN to END_TOKEN
      */
 
-    while ((pBuf = fgets(buf, XCMSDB_MAXLINELEN, stream)) != NULL) {
+    while ((fgets(buf, XCMSDB_MAXLINELEN, stream)) != NULL) {
 	if ((sscanf(buf, "%s", token)) && (strcmp(token, END_TOKEN) == 0)) {
 	    /*
 	     * Found END_TOKEN so break out of for loop
@@ -709,7 +721,7 @@ LoadColornameDB(void)
 {
     int size;
     FILE *stream;
-    char *pathname;
+    const char *pathname;
     struct stat txt;
     int length;
 
@@ -735,10 +747,17 @@ LoadColornameDB(void)
     }
 
     if ((stream = _XFopenFile (pathname, "r")) == NULL) {
+	/* can't open file */
+	XcmsColorDbState = XcmsDbInitFailure;
 	return(XcmsFailure);
     }
 
-    stringSectionSize(stream, &nEntries, &size);
+    if (stringSectionSize(stream, &nEntries, &size) != XcmsSuccess ||
+	nEntries == 0) {
+	(void) fclose(stream);
+	XcmsColorDbState = XcmsDbInitFailure;
+	return(XcmsFailure);
+    }
     rewind(stream);
 
     strings = (char *) Xmalloc(size);
@@ -809,7 +828,7 @@ _XcmsCopyISOLatin1Lowered(
 
 /*
  *	NAME
- *		_XcmsResolveColorString - 
+ *		_XcmsResolveColorString -
  *
  *	SYNOPSIS
  */
@@ -865,7 +884,7 @@ _XcmsResolveColorString (
 	if (result_format != XcmsUndefinedFormat
 		&& pColor_exact_return->format != result_format) {
 	    /* need to be converted to the target format */
-	    return(XcmsConvertColors(ccc, pColor_exact_return, 1, 
+	    return(XcmsConvertColors(ccc, pColor_exact_return, 1,
 		    result_format, (Bool *)NULL));
 	} else {
 	    return(XcmsSuccess);
